#!/usr/bin/env python

import scipy.constants as C
import pylab as pl
import numpy as np
from scipy.integrate import trapz


Om = 0.31
Ol = 0.69
Or = 9.E-5
H0 = 68.


def get_rho_c(H):
    Mpc_to_km = C.mega * C.parsec * 1.E-3
    H_seconds = H / Mpc_to_km
    return 3 * H_seconds**2 / (8. * np.pi * C.G)

def get_Ha(x, Om=Om, Ol=Ol, Or=Or, H0=H0):
    Ok = 1. - Om - Ol - Or
    return H0 * np.sqrt(Om/x**3 + Or/x**4 + Ol + Ok*x**2)

def question_2():
    x = np.linspace(1.E-4, 10., 100)
    H = get_Ha(x)
    rho_c = get_rho_c(H)

    pl.figure()
    pl.loglog(x, H, 'r-')

    pl.figure()
    pl.loglog(x, rho_c, 'k-')
    

def question_3():
    x = np.linspace(1.E-4, 10., 100)
    rho_c = get_rho_c(H0)
    rho_r = Or * rho_c * x**-4
    rho_m = Om * rho_c * x**-3
    rho_l = Ol * rho_c * x**0

    pl.figure()
    pl.loglog(x, rho_r, 'r--', label='radiation')
    pl.loglog(x, rho_m, 'b-', label='matter')    
    pl.loglog(x, rho_l, 'k-.', label='vacuum energy')
    pl.legend(loc='upper center')

    pl.axvline(1, ls=':')
    pl.axvline(1/3444., ls=':')
    pl.axvline(1/1.3, ls=':')
    
def question_4():
    x = np.logspace(-6, 1, 100)
    rho_c_0 = get_rho_c(H0)
    rho_r = Or * rho_c_0 * x**-4
    rho_m = Om * rho_c_0 * x**-3
    rho_l = Ol * rho_c_0 * x**0

    H = get_Ha(x)
    rho_c = get_rho_c(H)

    pl.figure()
    pl.semilogx(x, rho_r/rho_c, 'r--', label='radiation')
    pl.semilogx(x, rho_m/rho_c, 'b-', label='matter')
    pl.semilogx(x, rho_l/rho_c, 'k-.', label='vacuum energy')

    pl.xlabel('$a/a_0$')
    pl.ylabel('$\Omega_X(a)$')
    pl.legend(loc='best')

    pl.axvline(1/3444., ls=':')
    pl.axvline(1/1.3, ls=':')
    pl.axvline(1., ls=':')


def integrand(x, Om=Om, Ol=Ol, Or=Or, H0=H0):

    Ok = 1. - Om - Ol - Or
    r = H0 * np.sqrt(Om/x + Or/x**2 + Ol*x**2 + Ok)
    r = r / (C.mega * C.parsec * 1.E-3) * (C.giga * C.year)
    return 1 / r

def get_age(z, N=1000, Om=Om, Ol=Ol, Or=Or, H0=H0):

    xmax = 1/(1.+z)
    x_grid = np.linspace(1.E-6, xmax, 100)
    dt = integrand(x_grid, Om=Om, Ol=Ol, Or=Or, H0=H0)
    return trapz(dt, x=x_grid)

print(get_age(0.1))
print(get_age(0.3))
print(get_age(0.7))
print(get_age(1.0))

def  question_7():
    """
    """
    z = np.linspace(0., 5., 100)
    r = []
    for zz in z:
        r.append(get_age(zz))

    pl.figure()
    pl.plot(z, r, 'k-', label='$\Lambda CDM$'),
    
    r = []
    for zz in z:
        r.append(get_age(zz, Om=1., Ol=0, Or=0.))
    pl.plot(z, r, 'r-', label='$\Omega_m =1$'),

    pl.legend(loc='best')

    
